#import "ReminderController.h"
#import "Reminder.h"

@implementation ReminderController

- (void)applicationDidFinishLaunching:(NSNotification *)notification
{
    SEL clickSelector, timerSelector;

    clickSelector = @selector(updateControls);
    timerSelector = @selector(reloadData);
    reminderArray = [[NSMutableArray alloc] init];
    [remindersTable setDataSource:self];
    [remindersTable sizeLastColumnToFit];
    [remindersTable setAction:clickSelector];
    [NSTimer scheduledTimerWithTimeInterval:1 target:remindersTable selector:timerSelector userInfo:nil repeats:YES];
}

- (int)numberOfRowsInTableView:(NSTableView *)theTableView
{
    return [reminderArray count];
}

- (id)tableView:(NSTableView *)theTableView objectValueForTableColumn: (NSTableColumn *)theColumn row:(int)rowIndex;
{
    NSTimeInterval aTimeInterval;
    int hours, minutes, seconds;
    if ([[theColumn identifier] isEqualToString:@"Reminders"])
      {
        return [[reminderArray objectAtIndex:rowIndex] stringValue];
      }
    else if ([[theColumn identifier] isEqualToString:@"Time"])
      {
        aTimeInterval = [[reminderArray objectAtIndex:rowIndex] remainingTime];
        if(aTimeInterval == 0)
          {
            return @"Expired";
          }
        else
          {
            ConvertSecondsToTime(aTimeInterval, &hours, &minutes, &seconds);
            return [NSString stringWithFormat:@"%02d:%02d:%02d",hours,minutes,seconds];
          }
      }
    else if ([[theColumn identifier] isEqualToString:@"Repeats"])
      {
        if([[reminderArray objectAtIndex:rowIndex] getRepeats])
          {
            return @"Yes";
          }
        else
          {
            return @"No";
          }
      }
    else
      {
        return nil;
      }
}

- (void)updateControls
{
    int rowIndex;
    NSTimeInterval aTimeInterval;
    int hours, minutes, seconds;
    rowIndex = [remindersTable selectedRow];
    if([reminderArray count] < 1)
      {
        [deleteButton setEnabled:NO];
        [changeButton setEnabled:NO];
        [reminderMessageField setStringValue:@""];
      }
    else if(rowIndex > -1 && rowIndex < [reminderArray count])
      {
        [deleteButton setEnabled:YES];
        [changeButton setEnabled:YES];
        if([[reminderArray objectAtIndex:rowIndex] getValid])
          {
            [reminderInField setStringValue:[[reminderArray objectAtIndex:rowIndex] stringValue]];
            [reminderMessageField setStringValue:@""];
            aTimeInterval = [[reminderArray objectAtIndex:rowIndex] remainingTime];
            ConvertSecondsToTime(aTimeInterval, &hours, &minutes, &seconds);
            [hoursField setIntValue:hours];
            [minutesField setIntValue:minutes];
            [secondsField setIntValue:seconds];
            [hoursSlider setIntValue:hours];
            [minutesSlider setIntValue:minutes];
            [secondsSlider setIntValue:seconds];
            [switchBox setState:[[reminderArray objectAtIndex:rowIndex] getRepeats]];
         }
        else
         {
            [hoursField setIntValue:0];
            [minutesField setIntValue:0];
            [secondsField setIntValue:0];
            [hoursSlider setIntValue:0];
            [minutesSlider setIntValue:0];
            [secondsSlider setIntValue:0];
         }
      }
}

- (void)updateReminder:(id)sender
{
    Reminder *aReminder;
    int selectedRow = [remindersTable selectedRow];
    if (selectedRow > -1 && selectedRow < [reminderArray count])
      {
        aReminder = [reminderArray objectAtIndex:selectedRow];
        if(aReminder)
          {
            if([self addReminder:self atIndex:selectedRow])
              {
                [self deleteReminderIdenticalTo:aReminder];
              }
          }
      }
    else
      {
        NSBeep();
        NSRunAlertPanel(@"EggTimer Warning",@"Nothing to change!", @"OK", nil, nil);
        [remindersTable reloadData];
      }
}

- (void)addReminder:(id)sender
{
    [self addReminder:sender atIndex:-1];
}

- (BOOL)addReminder:(id)sender atIndex:(int)rowIndex
{
    Reminder *aReminder;
    SEL aSelector;
    int hr, min, sec;
    long totalSeconds;
    aSelector = @selector(showReminderPanel:);
    aReminder = [[Reminder alloc] init];
    hr = [hoursField intValue];
    min = [minutesField intValue];
    sec = [secondsField intValue];
    totalSeconds = ConvertTimeToSeconds(hr, min, sec);
    if(totalSeconds > 0)
      {
        [aReminder setReminder:totalSeconds :self :aSelector :(BOOL)repeats :[reminderInField stringValue]];
        if(rowIndex > -1)
          {
            [reminderArray insertObject:aReminder atIndex:rowIndex];
          }
        else
          {
            [reminderArray addObject:aReminder];
          }
        
        [remindersTable reloadData];
        [self updateControls];
        return YES;
      }
    else
      {
        NSBeep();
        NSRunAlertPanel(@"EggTimer Warning",@"Delay time must be greater than zero!", @"OK", nil, nil);
        [self updateControls];
        return NO;
      }
}

- (void)deleteReminderIdenticalTo:(id)aReminder
{
    [aReminder cancelReminder];
    [reminderArray removeObjectIdenticalTo:aReminder];
    [remindersTable reloadData];
    [self updateControls];
}

- (void)deleteSelectedReminder:(id)sender
{
    int rowIndex = [remindersTable selectedRow];      
    if(rowIndex > -1)
      {
        [[reminderArray objectAtIndex:rowIndex] cancelReminder];
        [reminderArray removeObjectAtIndex:rowIndex];
      }
    else
      {
        NSBeep();
        NSRunAlertPanel(@"EggTimer Warning",@"Nothing to delete!", @"OK", nil, nil);
      }

    if(rowIndex == 0)
      {
        rowIndex = 1;
      }
    [remindersTable selectRow:(rowIndex-1) byExtendingSelection:NO];
    [remindersTable reloadData];
    [self updateControls];
}

- (void)setRepeat:(id)sender
{
    if([sender state])
      {
        repeats = YES;
      }
    else
      {
        repeats = NO;
      }
}

- (void)showReminderPanel:(NSTimer *)sender
{
    Reminder *aReminder;
    aReminder = [sender userInfo];
    NSBeep();
    switch (NSRunAlertPanel(@"EggTimer Reminder", [aReminder stringValue], @"OK", @"Delete", nil))
      {
        case 0:
            [self deleteReminderIdenticalTo:aReminder];
            break;
      }
    [self updateControls];
}

long ConvertTimeToSeconds(int hr, int min, int sec)
{
    long seconds;
    seconds = ((hr *3600) + (min * 60) + sec);
    return seconds;
}

void ConvertSecondsToTime(long totalSeconds, int *hours, int *minutes, int *secs)
{
    *hours = totalSeconds/3600;
    *minutes = ((totalSeconds%3600)/60);
    *secs = ((totalSeconds%60));
}
@end
